#include "NBody.hpp"

#define WIDTH 600
#define HEIGHT 600

#define DRAW_ONLY_SELECTED_PARTICLE 49	// 1 Key 
#define DRAW_POINTS_NOT_SPHERES		50	// 2 Key 
#define DRAW_LANDMARKS				51	// 3 Key 

bool draw_only_selected_particle = false;
bool draw_points_not_spheres = true;
bool draw_landmarks = false;

const int NDIM = 3;					// The dimension of the world (2D, 3D ...);
const int N_STEPS = 5000;			// The maximum number of steps for a chosen particle
									// to draw it's orbit;

const float DEG2RAD = 3.14159/180;	// PI/180;

//-----------------------------------------
// VARIABLES FOR FONTS:

void *font = GLUT_BITMAP_HELVETICA_12;
void *fonts[] =
{
	GLUT_BITMAP_TIMES_ROMAN_24,
	GLUT_BITMAP_9_BY_15,
  	GLUT_BITMAP_TIMES_ROMAN_10,
 	GLUT_BITMAP_TIMES_ROMAN_24,
 	GLUT_BITMAP_HELVETICA_18,
	GLUT_BITMAP_HELVETICA_12,
	GLUT_BITMAP_HELVETICA_10
};

//-----------------------------------------
// GENERAL USE VARIABLE:

float raspect;							// The ratio for resizing the window;
int object= -1;							// object represents the number of the selected particle; 
										// if no particle was selected, then object == -1, else it 
										// retains the number of the selected particle;
double part_orbit[N_STEPS][NDIM];		// Retains the positions of a chosen particle from the
										// beginning till current time and it's used for drawing
										// the orbit of that particle;
int part_ID = 11;						// The id of the particle for which we will trace it's
										// orbit;
int curr_step= 0;						// Used to determine the current step of the chosen 
										// particle. If it exceeds the maximum limit of steps to
										// be traced (N_STEPS), it will be reseted to zero again
										// and again (done for memory reasons: can retain infinite
										// steps -> memory consumption);				



float xpos = 0, ypos = 3, zpos = 10;	// Variables used for moving camera;
float xrot = 0, yrot = 0;
float lastx = 0, lasty = 0;

bool mouse_move= false;					// If false, you cannot move the camera according to the
										// mouse movement; if true then you can;											

extern int numBodies;					// Defined in SetupNBody.cpp;
extern void* me;						// Defined in NBody_main.cpp;
extern clock_t start_clock;				// Defined in NBody_main.cpp;

//--------------------------------
// PROTOTIPE FUNCTIONS

void GLInit();
void idleFunc();
void camera (void);
void reshape(int width, int height);
void RenderScene();
void mouseMovement(int x, int y);
void mouseClick(int button, int state, int x, int y);
void keyboard (unsigned char key, int x, int y);
void select_meniu(int mod);
void special_keyboard( int key, int x, int y);
void pick(int x, int y);


/////////////////////////////////// startOpenGLSimulation Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// This function is called to display the simulation in
// OpenGL. If "display" variable is not TRUE then the
// simulation in OpenGL will not take place.
//
/////////////////////////////////// startOpenGLSimulation Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void startOpenGLSimulation(int argc, char * argv[])
{
	char * windowName = (char*)malloc(1024);

	sprintf (windowName, "NBody simulation : %d particles", numBodies);
	

	if(displayOpenGL)
    {
        // Run in  graphical window if requested 
        glutInit(&argc, argv);

		glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
        glutInitWindowPosition(100,10);
        glutInitWindowSize(WIDTH,HEIGHT); 
		glutCreateWindow(windowName); 

		// Enter full-screen mode
		glutFullScreen();

		GLInit(); 
        

		// Registering callback functions:
		glutDisplayFunc(RenderScene);			// The drawing scene function;
		glutReshapeFunc(reshape);				// If the window is reshaped;
		glutSpecialFunc(special_keyboard); 		// For using Up/Down Arrow etc.;
		glutKeyboardFunc(keyboard); 			// For using usual keys;
		glutPassiveMotionFunc(mouseMovement); 	// Check for mouse movement;
		glutMouseFunc(mouseClick);				// Check for mouse click;
		glutIdleFunc(idleFunc);					// Idle function;

		glutCreateMenu(select_meniu);
		glutAddMenuEntry("[Key 1] draw_only_selected_particle", 1);
		glutAddMenuEntry("[Key 2] draw_points_not_spheres", 2);
		glutAddMenuEntry("[Key 3] draw_landmarks", 3);
		glutAttachMenu(GLUT_RIGHT_BUTTON);
	
		free(windowName);

		try {
			 glutMainLoop();					// Start the program loop;
		}
		catch( char * str ) {
			std::cout << "OpenGL exited." << std::endl;
		}
	}else{
		free(windowName);
	}
}

/////////////////////////////////// GLInit Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Initialize OpenGL related stuff.
//
/////////////////////////////////// GLInit Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void GLInit()
{
    glClearColor(0.0 ,0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT);
    glClear(GL_DEPTH_BUFFER_BIT);

	glCullFace(GL_BACK);		// Don't draw the back sides of polygons;
	glEnable(GL_CULL_FACE);		// Turn on culling. We are going to enable backface culling.
								// It basically means that we don't draw 2 side polygons. 
								// The only side drawn is the side with the normal coming 
								// out of it.By enabling backface culling, it cuts down our 
								// fill rate bottleneck. Since our world is composed of 2D 
								// boxes, we will never see the inside of the boxes, so there 
								// is no need to draw them.  We can change GL_BACK to GL_FRONT 
								// for an interesting effect;
}

///////////////////////////////// drawCircle Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Draw one of the xxx circles
/////
///////////////////////////////// drawCircle Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void drawCircle(float radius)
{
	glBegin(GL_LINE_LOOP);
   		for (int i=0; i < 360; i++) {
			float degInRad = i*DEG2RAD;
      		glVertex3f(cos(degInRad)*radius, 0, sin(degInRad)*radius);
   		}
	glEnd();
}

///////////////////////////////// drawLandMarks Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Draw the coordinates axis, the circles, and the cube
/////
///////////////////////////////// drawLandMarks Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void drawLandMarks()
{
	glPushMatrix();	
		glTranslatef( 0.0f, 0.0f, 0.0f );
		// Y axis:
		glColor3f (0.0, 0.0, 1.0);
		glBegin(GL_LINES);
        	glVertex3f(0.0f, 0.0f, 0.0f);
            glVertex3f(0.0f, 5.0f, 0.0f);
        glEnd();
        // X axis:
        glColor3f (1.0, 0.0, 0.0);
        glBegin(GL_LINES);
         	glVertex3f(0.0f, 0.0f, 0.0f);
            glVertex3f(5.0f, 0.0f, 0.0f);
        glEnd();
        // Z axis:
        glColor3f (0.0, 1.0, 0.0);
        glBegin(GL_LINES);
         	glVertex3f(0.0f, 0.0f, 0.0f);
            glVertex3f(0.0f, 0.0f, 5.0f);
        glEnd();
  		// The concentric circles:      
        glColor3f (0.4, 0.1, 0.1);
       	for (float r=1; r<10; r++)
        	drawCircle(r);
        
        // Draw the big wired cube:
        glutWireCube(20);
	glPopMatrix();
}

/////////////////////////////////// displayfunc Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// This function is actually drawing all the particles in the system
// and calls for the "landmarks" drawing in the scene.
//
/////////////////////////////////// displayfunc Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void displayfunc()
{
    glClearColor(0.0 ,0.0, 0.0, 0.0);
 
    glPointSize(2.0);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glEnable(GL_BLEND);
    glDepthMask(GL_FALSE);

	// If we choose to draw the "landmarks":
	if (draw_landmarks)
		drawLandMarks();				// Draw the axis and other orientative stuff;
 
	//------------------------
	// Run the kernel on the choosen device:
	if (((NBody*)me)->curr_time_step < ((NBody*)me)->end_time){
		//Calling kernel for calculatig subsequent positions
		if (integrator == LEAPFROG){
			((NBody*)me)->runCLKernelLeapfrog();
		}else{
			((NBody*)me)->runCLKernels();
		}
	}

	//------------------------
	// After running the kernel we have the new updated positions:

	glInitNames();					// Initialize name stack needed for selection;
	glPushName(0);
	
	int contor= 0;					// Used to name objects (particles);

	// Draw the chosen particle's orbit:
	glColor3f(0.0f,1.0f,0.0f);
	glBegin(GL_LINE_STRIP);
		for (int i=0; i<curr_step; i++){
			glVertex3f(part_orbit[i][0], part_orbit[i][1], part_orbit[i][2]);
				
		}
	glEnd();

	// If we choose to draw the particles as points:
	if (draw_points_not_spheres){

		glBegin(GL_POINTS);
			for(int i=0; i < numBodies; ++i)
			{
				// If we choose to draw only the selected particle,
				// then wait for i to become that particle:
				if (draw_only_selected_particle)
					if (i != part_ID)
						continue;

				if (i == part_ID){						// If it is the chosen particle;

    				if (curr_step == N_STEPS)			// If curr_step exceeds the max limit
	    				curr_step= 0;					// of steps to be traced ==> reset;

	        		part_orbit[curr_step][0]= ((NBody*)me)->pos[i*4+ 0]/300;
					part_orbit[curr_step][1]= ((NBody*)me)->pos[i*4+ 1]/300;
					part_orbit[curr_step][2]= ((NBody*)me)->pos[i*4+ 2]/300;
					
					curr_step++;						// Update steps for the chosen particle;
				}

				if (i == part_ID)
					glColor3f(0.0f,1.0f,0.0f);
				else
					glColor3f(1.0f,0.6f,0.0f);

				// Divided by 300 just for scaling
				glVertex3d(	((NBody*)me)->pos[i*4+0]/300,
							((NBody*)me)->pos[i*4+1]/300,
							((NBody*)me)->pos[i*4+2]/300);
			}
		glEnd();
	
	} else {
	// Else the particles will be drawn as spheres:
		for(int i=0; i < numBodies; ++i)
		{
			// If we choose to draw only the selected particle,
			// then wait for i to become that particle:
			if (draw_only_selected_particle)
				if (i != part_ID)
					continue;

			if (i == part_ID){						// If it is the chosen particle;

    			if (curr_step == N_STEPS)			// If curr_step exceeds the max limit
	    			curr_step= 0;					// of steps to be traced ==> reset;

	        	part_orbit[curr_step][0]= ((NBody*)me)->pos[i*4+ 0]/300;
				part_orbit[curr_step][1]= ((NBody*)me)->pos[i*4+ 1]/300;
				part_orbit[curr_step][2]= ((NBody*)me)->pos[i*4+ 2]/300;
				
				curr_step++;						// Update steps for the chosen particle;
			}
			
			glLoadName(contor++);					// Register a particle for picking process;

			glPushMatrix();
				//divided by 300 just for scaling
				glTranslatef(((NBody*)me)->pos[i*4+ 0]/300,
							((NBody*)me)->pos[i*4+1]/300,
							((NBody*)me)->pos[i*4+2]/300);

				if (i == part_ID)
					glColor3f(0.0f,1.0f,0.0f);
				else
					glColor3f(1.0f,0.6f,0.0f);

				glutSolidSphere(0.01, 10, 10);
			glPopMatrix();
		}
	}
}

///////////////////////////////// RenderScene Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Will prepare the scene for drawing, then it will set the camera to
///// the right coordinates and in the end will call for the drawing func.
/////
///////////////////////////////// RenderScene Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void RenderScene() 
{
	// Clear the window with current clearing color, 
	// and also the depth buffer :
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// Clear the matrix :
	glLoadIdentity ();  
	
	// Set the camera to the new position:
	camera();
	
	
	// Draw the scene:
	displayfunc();

    glFlush();
    glutSwapBuffers();

}

/////////////////////////////////// idle Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Glut Idle function.
//
/////////////////////////////////// idle Func \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void idleFunc()
{
	// If it's time to take a snapshot:
	if (takeSnapshots == TAKE_SNAPSHOTS){
		if (((NBody*)me)->curr_time_step > ((NBody*)me)->curr_snap_time){
			((NBody*)me)->curr_snap_time += ((NBody*)me)->dt_snap;
			((NBody*)me)->put_snapshot();
		}
	}

	// If it's time to take a diagnostication:
	if (takeDiagnostics == TAKE_DIAGNOSTICS){
		if (((NBody*)me)->curr_time_step > ((NBody*)me)->curr_diag_time){
			((NBody*)me)->curr_diag_time += ((NBody*)me)->dt_diag;
			((NBody*)me)->write_diagnostics();
			//std::cout << "Current time: " << ((NBody*)me)->curr_time_step << std::endl;
		}

		// If it's the first step in the simulation compute
		// the initial total energy of the system:
		if (((NBody*)me)->taken_steps == 0){
			((NBody*)me)->write_diagnostics();
		}
	}

		if (((NBody*)me)->curr_time_step > ((NBody*)me)->curr_diag_time){
			((NBody*)me)->curr_diag_time += ((NBody*)me)->dt_diag;
			std::cout << "Current time: " << ((NBody*)me)->curr_time_step << std::endl;
		}

	// If it is not the end of the simulation:
	if (((NBody*)me)->curr_time_step < ((NBody*)me)->end_time){
		
		// Move one step foreward (a step = delT):
		((NBody*)me)->curr_time_step += ((NBody*)me)->delT;
		
		// Increase the number of steps by now:
		((NBody*)me)->taken_steps++;
		
		//if (((NBody*)me)->taken_steps == 1800)
		//	std::cout << "some test beep here \a\n";

		//std::cout << 

	}else{
		std::cout << std::endl;
		std::cout << "Simulation ended..." << std::endl;
		std::cout << std::endl;

		clock_t finish_clock = clock();

		std::cout << "Runtime : " << (double)(finish_clock - start_clock) / CLOCKS_PER_SEC << std::endl;
		std::cout << "============================" << std::endl;
		
		exit(1);
	}

    glutPostRedisplay();
}

//////////////////////////// RESIZE OPENGL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function resizes the viewport for OpenGL.
/////   @param w width of OpenGL window
/////   @param h height of OpenGL window 
/////
//////////////////////////// RESIZE OPENGL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void reshape(int width, int height)		
{
	if (height==0)		// Prevent A Divide By Zero error
	{
		height=1;										
	}

	glViewport(0,0,width,height);					
														

	glMatrixMode(GL_PROJECTION);					
	glLoadIdentity();
	
	raspect= (GLfloat)width/(GLfloat)height;

	gluPerspective(60.0f,raspect, 0.5 ,1250.0f);
	
	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();
}

///////////////////////////////// camera \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Move and rotate camer according to the user interaction
/////
///////////////////////////////// camera \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void camera (void) 
{
	glRotatef(xrot,1.0,0.0,0.0);  	// Rotate camera on teh x-axis (left and right);
	glRotatef(yrot,0.0,1.0,0.0);  	// Rotate camera on the y-axis (up and down);
	glTranslated(-xpos,-ypos,-zpos);// Translate the screen to the position of the camera;
}

//////////////////////////// KEYHIT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Here are implemented the special Keys:
/////	Up/Down Arrows etc. 
/////
//////////////////////////// KEYHIT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void special_keyboard( int key, int x, int y)
{
	switch(key) 
	{
		case GLUT_KEY_UP    : 	// MOVE FORWARD:
								float xrotrad, yrotrad;
							  	yrotrad = (yrot / 180 * 3.141592654f);
							  	xrotrad = (xrot / 180 * 3.141592654f); 
							  	xpos += float(sin(yrotrad)/5);
							  	zpos -= float(cos(yrotrad)/5);
						      	ypos -= float(sin(xrotrad)/5);
							  	glutPostRedisplay(); 
							  	break;
		case GLUT_KEY_DOWN  : 	// MOVE BACKWARD:
								yrotrad = (yrot / 180 * 3.141592654f);
							 	xrotrad = (xrot / 180 * 3.141592654f); 
							  	xpos -= float(sin(yrotrad)/5);
							  	zpos += float(cos(yrotrad)/5);
							  	ypos += float(sin(xrotrad)/5);
							  	glutPostRedisplay(); 
							  	break;
		case GLUT_KEY_LEFT  : 	// MOVE LEFT:
								yrotrad = (yrot / 180 * 3.141592654f);
							  	xpos -= float(cos(yrotrad)/5) * 0.2;
							  	zpos -= float(sin(yrotrad)/5) * 0.2;
							  	glutPostRedisplay(); 
							  	break;
		case GLUT_KEY_RIGHT : 	// MOVE RIGHT:
								yrotrad = (yrot / 180 * 3.141592654f);
							  	xpos += float(cos(yrotrad)/5) * 0.2;
							  	zpos += float(sin(yrotrad)/5) * 0.2;
							  	glutPostRedisplay(); 
							  	break;
		case GLUT_KEY_END   : 	if(((NBody*)me)->cleanup() != 0) // SDK_SUCCESS == 0;
									exit(1);
								else
									exit(0);
 							  	
	}
}

//////////////////////////// keyboard \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Here are implemented the normal use Keys
/////
//////////////////////////// keyboard \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void keyboard (unsigned char key, int x, int y) 
{
	switch(key){
		case 'w'   :	// MOVE FORWARD:
						float xrotrad, yrotrad;
						yrotrad = (yrot / 180 * 3.141592654f);
						xrotrad = (xrot / 180 * 3.141592654f); 
						xpos += float(sin(yrotrad)/5);
						zpos -= float(cos(yrotrad)/5);
						ypos -= float(sin(xrotrad)/5);
						glutPostRedisplay(); 
						break;
		case 's'  : 	// MOVE BACKWARD:
						yrotrad = (yrot / 180 * 3.141592654f);
						xrotrad = (xrot / 180 * 3.141592654f); 
						xpos -= float(sin(yrotrad)/5);
						zpos += float(cos(yrotrad)/5);
						ypos += float(sin(xrotrad)/5);
						glutPostRedisplay(); 
						break;
		case 'a'  : 	// MOVE LEFT:
						yrotrad = (yrot / 180 * 3.141592654f);
						xpos -= float(cos(yrotrad)/5) * 0.2;
						zpos -= float(sin(yrotrad)/5) * 0.2;
						glutPostRedisplay(); 
						break;
		case 'd' : 		// MOVE RIGHT:
						yrotrad = (yrot / 180 * 3.141592654f);
						xpos += float(cos(yrotrad)/5) * 0.2;
						zpos += float(sin(yrotrad)/5) * 0.2;
						glutPostRedisplay(); 
						break;
		case 'q' :		// MOVE DOWN (Y):
						ypos-= 1;
						glutPostRedisplay(); 
						break;
		case 'e' :		// MOVE UP (Y):
						ypos+= 1;
						glutPostRedisplay(); 
						break;

		case 'm' :		// MOVE CAMERA WITH MOUSE:
						if (mouse_move == false)
							mouse_move= true;
						else
							mouse_move= false;
						glutPostRedisplay(); 
						break;

		case DRAW_ONLY_SELECTED_PARTICLE :	
						// 1 Key 
						if (draw_only_selected_particle)
							draw_only_selected_particle = false;
						else
							draw_only_selected_particle = true;
						glutPostRedisplay();
						break;

		case DRAW_POINTS_NOT_SPHERES :	
						// 2 Key 
						if (draw_points_not_spheres)
							draw_points_not_spheres = false;
						else
							draw_points_not_spheres = true;
						glutPostRedisplay();
						break;
		
		case DRAW_LANDMARKS	: 
						// 3 Key
						if (draw_landmarks)
							draw_landmarks = false;
						else
							draw_landmarks = true;
						glutPostRedisplay();
						break;

		case 27 :
            {
                if(((NBody*)me)->cleanup() != 0) // SDK_SUCCESS == 0;
                    exit(1);
                else
                    exit(0);
            }
        default :
            break;

					
	}
}

void select_meniu(int mod)
{
	switch (mod) {
		case 1	:	if (draw_only_selected_particle)
						draw_only_selected_particle = false;
					else
						draw_only_selected_particle = true;
					glutPostRedisplay();
					break;
		case 2	:	if (draw_points_not_spheres)
						draw_points_not_spheres = false;
					else
						draw_points_not_spheres = true;
					glutPostRedisplay();
					break;
		case 3	:	if (draw_landmarks)
						draw_landmarks = false;
					else
						draw_landmarks = true;
					glutPostRedisplay();
					break;
	}
}


//////////////////////////// mouseMovement \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Here we trace the mouse movement and use this
///// to move the camera
/////
//////////////////////////// mouseMovement \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void mouseMovement(int x, int y) 
{
	if (mouse_move == true){
		int diffx=x-lastx; 		// Check the difference between the current x 
								// and the last x position;
		int diffy=y-lasty; 		// Check the difference between the current y 
								// and the last y position;
		lastx=x; 				// Set lastx to the current x position;
		lasty=y; 				// Set lasty to the current y position;
		xrot += (float) diffy; 	// Set the xrot to xrot with the addition of 
								// the difference in the y position;
		yrot += (float) diffx;	// Set the xrot to yrot with the addition of 
								// the difference in the x position;
	}
}

//////////////////////////// mouseClick \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Here we determine if a mouse button was clicked and try
///// to find on which particle it was clicked
/////		
//////////////////////////// mouseClick \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void mouseClick(int button, int state, int x, int y){
	
	switch(button){
		case GLUT_LEFT_BUTTON:	// if we pressed left button :

								if(state == GLUT_DOWN ){
									// it is done the selection, here, where the click
									// took place :
									pick(x,y);
									// the animation is started :
									glutIdleFunc(idleFunc);
								}
								break;
		case GLUT_RIGHT_BUTTON:	if(state== GLUT_DOWN)
								//	glutIdleFunc(NULL);
								glutIdleFunc(idleFunc);
								break;
	}
}	

///////////////////////////////// processhits \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Here are precessed the hits from selecting objects (particles)
/////
///////////////////////////////// processhits \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void processhits (GLint hits, GLuint buffer[]){
	/*
	 * After we determined that there were hits, we find out which 
	 * particles were selected an put them into "object" variable;
	 */
   int i;
   GLuint names, *ptr, minZ,*ptrNames, numberOfNames;

   ptr = (GLuint *) buffer;
   minZ = 0xffffffff;
   for (i = 0; i < hits; i++) {	
      names = *ptr;
	  ptr++;
	  if (*ptr < minZ) {
		  numberOfNames = names;
		  minZ = *ptr;
		  ptrNames = ptr+2;
	  }
	  
	  ptr += names+2;
  }

  ptr = ptrNames;
  
  object = *ptr;
}

///////////////////////////////// pick \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// Determines which particle was selectet with the mouse
/////
///////////////////////////////// pick \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
void pick(int x, int y){
	/*
	 * This is a standard function that resolv the problem of selecting an object.
	 * After we determine if there was at least a hit, we find which particle was
	 * selected and then make it the traced particle for whom we will draw its
	 * orbit.
	 */

	GLuint buffer[1024];
	GLint nhits;

	GLint	viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);
	glSelectBuffer(1024, buffer);
	
	// Entering in selection mode :
	glRenderMode(GL_SELECT);

	// It is rendered just a little area around the mouse cursor : 
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
		glLoadIdentity();

		glGetIntegerv(GL_VIEWPORT,viewport);
		gluPickMatrix(x,viewport[3]-y,0.2f,0.2f,viewport);
		
		// It's the same like in reshape function:
		gluPerspective(60.0f,raspect, 0.5 ,1250.0f);
		glMatrixMode(GL_MODELVIEW);

		// The scene is drawn in selection mode (in fact nothing 
		// is effectively drawn : the "drawing" is made in the 
		// selection buffer not on the screen): 
		 RenderScene();

		glMatrixMode(GL_PROJECTION);						
	glPopMatrix();								
	
	glMatrixMode(GL_MODELVIEW);						
	nhits=glRenderMode(GL_RENDER);	
	
	// Initially there is no selected object:
	object = -1;

	// Now we process the selection buffer and choose 
	// the nearest object (if it exists) :
	if(nhits != 0){
		processhits(nhits,buffer);	// process the buffer;
		//printf("selected particle %d\n",object);
		part_ID= object;
		curr_step= 0;

	}
}
